;;
;; Use Gnuplot to plot diagrams
;;

; First the original plot.scm, plotting data by rows

(define (hplot-cell fp buf row col)
  (if (= (get-type buf row col) EXPRESSION)
    (writes fp (get-cell row col) "\n")
    (writes fp "\n")))

; Check if the first row can be used for tics
; Yes, the heuristics are questionable
(define (is-tics buf row c1 c2)
  (let ((type (get-type buf row c1)))
    (and (<= c1 c2)
	 (or (= type LABEL)
	     (and (not (= type EXPRESSION))
		  (is-tics buf row (+ c1 1) c2))))))

; Check if the contents of the first column is likely to be titles
(define (is-titles buf r1 r2 col)
  (let ((type (get-type buf r1 col)))
    (and (<= r1 r2)
	 (or (= type LABEL)
	     (and (not (= type EXPRESSION))
		  (is-titles buf (+ r1 1) r2 col))))))

(define (hplot style)
  (let ((has-tics nil) (has-titles nil)
	(pid 0)
	(c 0) (c1 0) (c2 0) (r 0) (r1 0) (r2 0)
	(fn-cmd "") (fn-output "") (fn-data "")
	(fp-cmd nil) (fp-output nil) (fp-data nil))
    (set! pid (number->string (getpid) 10))
    (set! fn-cmd (string-append "/tmp/siagplot" pid ".cmd"))
    (set! fn-output (string-append "/tmp/siagplot" pid ".ps"))
    (set! fp-cmd (fopen fn-cmd "w"))
    (writes fp-cmd "# This file is used by Siag to control Gnuplot\n")
    (writes fp-cmd "set terminal postscript\n")
    (writes fp-cmd "set output \"" fn-output "\"\n")
    (writes fp-cmd "set data style " style "\n")
    (set! r0 (position-row (get-blku)))
    (set! r1 r0)
    (set! r2 (position-row (get-blkl)))
    (set! c0 (position-col (get-blku)))
    (set! c1 c0)
    (set! c2 (position-col (get-blkl)))
    ; the case with only one line must be treated specially
    (set! has-tics (if (= c1 c2) (is-tics nil r0 c1 c2)
				 (is-tics nil r0 (+ c1 1) c2)))
    (set! has-titles (if (= r1 r2) (is-titles nil r1 r2 c0)
				   (is-titles nil (+ r1 1) r2 c0)))
    (if has-titles (set! c1 (+ c1 1)))
    (if has-tics
      (begin
        (set! r1 (+ r1 1))
        (writes fp-cmd "set xtics (")
        (set! c c1)
        (while (<= c c2)
  	  (if (> c c1) (writes fp-cmd ", "))
	  (if (get-text r0 c)
	    (writes fp-cmd "\"" (get-string r0 c) "\" "))
	  (writes fp-cmd (- c c1))
	  (set! c (+ c 1)))
        (writes fp-cmd ")\n")))
    (writes fp-cmd "plot ")
    (set! r r1)
    (while (<= r r2)
      (set! fn-data (string-append "/tmp/siagplot" pid "." (number->string r 10)))
      (set! fp-data (fopen fn-data "w"))
      (writes fp-cmd "\"" fn-data "\"")
      (if (and has-titles (get-text r c0))
	(writes fp-cmd " title \"" (get-string r c0) "\""))
      (if (< r r2)
	(writes fp-cmd ", "))
      (set! c c1)
      (while (<= c c2)
	(hplot-cell fp-data nil r c)
	(set! c (+ c 1)))
      (fclose fp-data)
      (set! r (+ r 1)))
    (writes fp-cmd "\n")
    (fclose fp-cmd)
    (system "gnuplot " fn-cmd)
    (set! pid (spawn (string-append viewer-command " -landscape " fn-output)))
;    (deletia-add pid fn-data)
;    (deletia-add pid fn-cmd)
	))
;    (spawn (string-append "ghostview -landscape " fn-output))))

; End of horizontal plot

; ----------------------

; And now, plotting by column

;;
;; Use Gnuplot to plot diagrams
;;
;; Modified by Rodrigo Alfonso Guzman (10-2-1999) (raguzman@impsat1.com.ar)
;; so the selection is done by columns instead of by rows.


;(define (vplot-cell fp buf row col)
;  (if (= (get-type buf row col) EXPRESSION)
;    (writes fp (get-cell row col) "\n")
;    (writes fp "\n")))

(define (vplot-cell1 fp buf row col)
  (if (= (get-type buf row col) EXPRESSION)
    (writes fp (get-cell row col) "\n")
    (writes fp "\n")))

(define (vplot-cell fp buf row col rowx colx)
  (if (= (get-type buf row col) EXPRESSION)
    (writes fp (get-cell rowx colx) "\t" (get-cell row col) "\n" )
    (writes fp "\n")))

; Check if the first row can be used for tics
; Yes, the heuristics are questionable
;(define (is-tics buf row c1 c2)
;  (let ((type (get-type buf row c1)))
;    (and (<= c1 c2)
;	 (or (= type LABEL)
;	     (and (not (= type EXPRESSION))
;		  (is-tics buf row (+ c1 1) c2))))))
;

; Ver si la primera columna se puede usar para ticks.
; Check if the first row can be used for tics
; Yes, the heuristics are questionable
(define (is-tics buf col r1 r2)
  (let ((type (get-type buf r1 col)))
    (and (<= r1 r2)
	 (or (= type LABEL)
	     (and (not (= type EXPRESSION))
		  (is-tics buf col (+ r1 1) r2))))))



; ver si los contenidos de la primera fila son titulos.
; Check if the contents of the first column is likely to be titles
(define (is-titles buf c1 c2 row)
  (let ((type (get-type buf row c1)))
    (and (<= c1 c2)
	 (or (= type LABEL)
	     (and (not (= type EXPRESSION))
		  (is-titles buf (+ c1 1) c2 row))))))

(define (vplot style)
  (let ((has-tics nil) (has-titles nil)
	(pid 0)
	(c 0) (c1 0) (c2 0) (r 0) (r1 0) (r2 0)
	(fn-cmd "") (fn-output "") (fn-data "")
	(fp-cmd nil) (fp-output nil) (fp-data nil))
    (set! pid (number->string (getpid) 10))
    (set! fn-cmd (string-append "/tmp/siagplot" pid ".cmd"))
    (set! fn-output (string-append "/tmp/siagplot" pid ".ps"))
    (set! fp-cmd (fopen fn-cmd "w"))
    (writes fp-cmd "# This file is used by Siag to control Gnuplot\n")
    (writes fp-cmd "set terminal postscript\n")
    (writes fp-cmd "set output \"" fn-output "\"\n")
    (writes fp-cmd "set data style " style "\n")
    (set! r0 (position-row (get-blku)))
    (set! r1 r0)
    (set! r2 (position-row (get-blkl)))
    (set! c0 (position-col (get-blku)))
    (set! c1 c0)
    (set! c2 (position-col (get-blkl)))
    ; the case with only one line must be treated specially
    ;(set! has-tics (if (= c1 c2) (is-tics nil r0 c1 c2)
    ;				 (is-tics nil r0 (+ c1 1) c2)))
    (set! has-titles (if (= c1 c2) (is-titles nil c1 c2 r0)
				   (is-titles nil (+ c1 1) c2 r0)))
    (if has-titles (set! r1 (+ r1 1)))
 
    (writes fp-cmd "plot ")

; Rodrigo A. Guzman. (raguzman@impsat1.com.ar)
; Codigo modificado para dibujar tablas del tipo x y1 y2 y3....
; el bloque seleccionado debe tener 2 o mas columnas.
; c1  es la columna que tiene los valores de X

    (if (< c1 c2)
      (set! c (+ c1 1))    ; comienza con y1
      (set! c c1))         ; solo 1 vector.  

    (while (<= c c2)
      (set! fn-data (string-append "/tmp/siagplot" pid "." (number->string c 10)))
      (set! fp-data (fopen fn-data "w"))
      (writes fp-cmd "\"" fn-data "\"")
      (if (and has-titles (get-text r0 c))
	(writes fp-cmd " title \"" (get-string r0 c) "\""))
      (if (< c c2)
	(writes fp-cmd ", "))
      (set! r r1)
      (while (<= r r2)
        (if (< c1 c2)
	  (vplot-cell fp-data nil r c r c1)
          (vplot-cell1 fp-data nil r c))
	(set! r (+ r 1)))
      (fclose fp-data)
      (set! c (+ c 1))) 
;-----------------------------------------------------------------

    (writes fp-cmd "\n")
    (fclose fp-cmd)
    (system "gnuplot " fn-cmd)
    (set! pid (spawn (string-append viewer-command " -landscape " fn-output)))
;    (deletia-add pid fn-data)
;    (deletia-add pid fn-cmd)
	))
;    (spawn (string-append "ghostview -landscape " fn-output))))

; End of vplot.scm

; ----------------

; And now, glue it together

(define plot hplot)	; or vplot

; ----------------

; Plotting into a plugin

(define (plot-plugin style)
  (let* ((blku (get-blku))
	 (blkl (get-blkl))
	 (r 0)
	 (c 0)
	 (r1 (position-row blku))
	 (c1 (position-col blku))
	 (r2 (position-row blkl))
	 (c2 (position-col blkl))
	 (has-tics nil)
	 (xtics (if has-tics 1 0))
	 (has-titles nil)
	 (titles (if has-titles 1 0))
	 (horizontal 1)
	 (update 0)
  	 (pid (number->string (getpid) 10))
	 (fn (string-append "/tmp/siagplot" pid ".plot"))
	 (fp (fopen fn "w")))
    (writes fp "[Control]\n")
    (writes fp "range="
	       (number->string r1)
	       " "
	       (number->string c1)
	       " "
	       (number->string r2)
	       " "
	       (number->string c2)
	       "\n")
    (writes fp "style="
	       style
	       "\n")
    (writes fp "titles="
	       (number->string titles)
	       "\n")
    (writes fp "xtics="
	       (number->string xtics)
	       "\n")
    (writes fp "horizontal="
	       (number->string horizontal)
	       "\n")
    (writes fp "update="
	       (number->string update)
	       "\n")
    (writes fp "\n[Data]\n")
    (set! r r1)
    (while (<= r r2)
      (set! c c1)
      (while (<= c c2)
	(let ((type (get-type nil r c)))
      	  (cond
	    ((= type EXPRESSION)
	     (writes fp (number->string r)
			" "
			(number->string c)
			" "
			(number->string (get-cell r c))
			"\n"))
	    ((or (= type STRING) (= type LABEL))
	     (writes fp (number->string r)
		        " "
		        (number->string c)
		        " "
		        (get-string r c)
		        "\n"))
	    (t nil)))
	(set! c (+ c 1)))
      (set! r (+ r 1)))
    (fclose fp)
    (plugin-import fn)))

